	;**********************************************************************
	;                                                                     *
	;**********************************************************************
	;    Filename:	    oven.asm                                          *
	;    Date:                                                            *
	;    File Version:                                                    *
	;                                                                     *
	;    Author:                                                          *
	;    Company:                                                         *
	;                                                                     *
	;**********************************************************************
	;    Files Required: P16F677.INC                                      *
	;                                                                     *
	;**********************************************************************
	;    PIC 16F677 pinout:                                               *
	;                          |--\/--|                                   *
	;                     Vdd -|1   20|- Vss                              *
	;    RA5/T1CKI/OSC1/CLKIN -|2   19|- RA0/AN0/C1IN+/ICSPDAT/ULPWU      *
	; RA4/AN3/T1G/OSC2/CLKOUT -|3   18|- RA1/AN1/C12IN0-/VREF/ICSPCLK     *
	;            RA3/MCLR/VPP -|4   17|- RA2/AN2/T0CKI/INT/C1OUT          *
	;                     RC5 -|5   16|- RC0/AN4/C2IN+                    *
	;               RC4/C2OUT -|6   15|- RC1/AN5/C12IN1-                  *
	;           RC3/AN7C12IN3 -|7   14|- RC2/AN6/C12IN2-                  *
	;              RC6/AN8/SS -|8   13|- RB4/AN10/SDI/SDA                 *
	;             RC7/AN9/SDO -|9   12|- RB5/AN11                         *
	;                     RB7 -|10  11|- RB6/SCK/SCL                      *
	;                          --------                                   *
	;                                                                     *
	;**********************************************************************

;/****************************************************************************
;* DESCRIPTION: System definitions.
;* return:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

	ERRORLEVEL -302 ;remove message about using proper bank

	LIST  P=16F677
	INCLUDE <P16F677.INC>
	__CONFIG _BOR_OFF & _FCMEN_OFF & _IESO_OFF & _CPD_OFF & _CP_OFF & _MCLRE_OFF & _PWRTE_ON & _WDT_OFF & _INTOSCIO

;/****************************************************************************
;* DESCRIPTION: Program variables and constant definitions.
;* return:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

TEMPER		EQU 0x20						; Store temperature
TEMP		EQU 0x21						; General use variable
W_TEMP		EQU 0x22						; Temporary W register
STATUS_TEMP	EQU 0x23						; Temporary STATUS register
UNITS		EQU 0x24						; UNITS
TENS		EQU	0x25						; TENS
HUNDREDS	EQU 0x26						; HUNDREDS
AUX			EQU	0x27						; GENERAL AUX. REGISTER
TEMPO0		EQU	0x28
TEMPO1		EQU	0x29
T1_CONTA	EQU 0x2A						; Aux. Timer1 counter
TMR_1MS		EQU	0x2B						; Counter for 1 ms period
TMR_100MS	EQU	0x2C						; Counter for 100 ms period
FILTER_KEYS	EQU	0x2D
TIME_TURBO	EQU	0x2E
BEEPCNT		EQU	0x2F						; Beep cycle counter
DCNT0		EQU	0x30
DCNT1		EQU	0x31
FLAGS		EQU	0x32						; General use flags
FLAGS2		EQU	0x33
TIMER		EQU	0x34						; Timer register
BEEPTONE	EQU	0x35						; Buzzer tone register
BEEPLENGTH	EQU	0x36						; Buzzer tone lenght

#DEFINE		COOKINGMODE			FLAGS,0		; Cooking mode?
#DEFINE		PAUSEMODE			FLAGS,1		; Pause mode?
#DEFINE		TEMPMODE			FLAGS,2		; Temperature mode?
#DEFINE		TIMEMODE			FLAGS,3		; Time mode?
#DEFINE		LIGHTMODE			FLAGS,4		; Light ON?
#DEFINE		TOASTERMODE			FLAGS,5		; Toaster ON?
#DEFINE		SHOWTEMP			FLAGS,6		; Show temperatue?
#DEFINE		SHOWTIME			FLAGS,7		; Show timer?

#DEFINE		BUZZERON			FLAGS2,0	; Buzzer on?

; Leds
#DEFINE		LED_CMN				PORTC,2		; Leds Anodes

#DEFINE		LED_UP				PORTA,4		; Led UP
#DEFINE		LED_TIMER			PORTB,6		; Led TIMER
#DEFINE		LED_TEMP			PORTB,5		; Led TEMP
#DEFINE		LED_DWN				PORTA,2		; Led DWN
#DEFINE		LED_LIGHT			PORTB,4		; Led LIGHT
#DEFINE		LED_TOASTER			PORTA,5		; Led TOASTER
#DEFINE		LED_ONOFF			PORTC,6		; Led ONOFF
#DEFINE		TRIS_LED_ONOFF		TRISC,6		; Led ONOFF TRIS

; Displays
#DEFINE		DISP1				PORTC,3		; Anode display DIS0
#DEFINE		DISP2				PORTC,4		; Anode display DIS1
#DEFINE		DISP3				PORTC,5		; Anode display DIS2

; Keys
#DEFINE		KEY_UP				PORTB,7		; KEY_UP
#DEFINE		KEY_TIMER			PORTB,6		; KEY_TIMER
#DEFINE		KEY_ONOFF			PORTB,5		; KEY_ONOFF
#DEFINE		KEY_TEMP			PORTA,4		; KEY_TEMP
#DEFINE		KEY_DWN				PORTA,2		; KEY_DWN
#DEFINE		KEY_LIGHT			PORTB,4		; KEY_LIGHT
#DEFINE		KEY_TOASTER			PORTA,5		; KEY_TOASTER

; Buzzer
#DEFINE		BUZZER				PORTC,0		; Buzzer
#DEFINE		TRISBUZZER			TRISC,0		; Tris for buzzer

;/****************************************************************************
;* DESCRIPTION: Got Start of program.
;* return:      none
;* ALGORITHM:   none
;* NOTES:       none
;***************************************************************************** 
	ORG		0x0000
	goto	START

;*****************************************************************************
	ORG 0x0004
	goto	TIMER1_ISR

;/****************************************************************************
;* DESCRIPTION: Decodes value in W register to display segments for PORTA.
;* return:      result in W
;* ALGORITHM:   none
;* NOTES:
;
;       A
;	 _______
;   |       |
; F |       | B
;   |   G   |
;    -------
;   |       |
; E |       | C
;   |_______|
;       D
;*****************************************************************************/ 

DECOD_DISPLAY_A:
	addwf   PCL,F			; compute the jump value
TABLE_A:
						;	|7|6|5|4|3|2|1|0| -	Bits PORTA
						;	|-|-|B|E|-|A|-|-|
	retlw	B'00000000'	;	|-|-|B|E|-|A|-|-| -	0
	retlw	B'00010100'	;	|-|-|B|X|-|X|-|-| -	1
	retlw	B'00000000'	;	|-|-|B|E|-|A|-|-| -	2
	retlw	B'00010000'	;	|-|-|B|X|-|A|-|-| -	3
	retlw	B'00010100'	;	|-|-|B|X|-|X|-|-| -	4
	retlw	B'00110000'	;	|-|-|X|X|-|A|-|-| -	5
	retlw	B'00100000'	;	|-|-|X|E|-|A|-|-| -	6
	retlw	B'00010000'	;	|-|-|B|X|-|A|-|-| -	7
	retlw	B'00000000'	;	|-|-|B|E|-|A|-|-| -	8
	retlw	B'00010000'	;	|-|-|B|X|-|A|-|-| -	9


;/****************************************************************************
;* DESCRIPTION: Decodes value in W register to display segments, for PORTB.
;* return:      result in W
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

DECOD_DISPLAY_B:
	addwf   PCL,F			; compute the jump value

TABLE_B:
						;	|7|6|5|4|3|2|1|0| -	Bits PORTB
						;	|C|F|D|G|-|-|-|-| -
	retlw	B'00010000'	;	|C|F|D|X|-|-|-|-| -	0
	retlw	B'01110000'	;	|C|X|X|X|-|-|-|-| -	1
	retlw	B'11000000'	;	|X|X|D|G|-|-|-|-| -	2
	retlw	B'01000000'	;	|C|X|D|G|-|-|-|-| -	3
	retlw	B'00100000'	;	|C|F|X|G|-|-|-|-| -	4
	retlw	B'00000000'	;	|C|F|D|G|-|-|-|-| -	5
	retlw	B'00000000'	;	|C|F|D|G|-|-|-|-| -	6
	retlw	B'01110000'	;	|C|X|X|X|-|-|-|-| -	7
	retlw	B'00000000'	;	|C|F|D|G|-|-|-|-| -	8
	retlw	B'00000000'	;	|C|F|D|G|-|-|-|-| -	9

;/****************************************************************************
;* DESCRIPTION: Interrupt sevice.
;* return:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

TIMER1_ISR:
	movwf	W_TEMP			; save W content to W_TEMP
	swapf	STATUS,W		; put STATUS in W...
	movwf	STATUS_TEMP		; ... Then save it in STATUS_TEMP

	bcf		STATUS,RP0		; Bank 0
	bcf		STATUS,RP1		;

	bcf		PIR1,TMR1IF		; Clear Timer1 Interrupt flag
	movlw	0x18			;
	movwf	TMR1L			;
	movlw	0xFC			;
	movwf	TMR1H			; Init TIMER1 with decimal 64536 (interrupt every 1ms)
	incf	TMR_1MS,F		; Increment TR_1MS (1 ms counter)
	movlw	D'100'
	xorwf	TMR_1MS,W		; Is TMR_1MS = 100?
	btfsc	STATUS,Z		; No, skip if TMR_1MS < 100
	goto	INC100MS		; Yes, zero TMR_1MS and inc TMR_100MS
	goto	END_INT			; No, exit interrupt
INC100MS
	movlw	D'0'			;
	movwf	TMR_1MS			; Zero TMR_1MS
	incf	TMR_100MS,F		; Increment TMR_100MS (100 ms counter)
	movlw	D'5'			; 500 ms?
	xorwf	TMR_100MS,W		; Is TMR_100MS = 5 (500ms)?
	btfsc	STATUS,Z		; No, skip if TMR_100MS < 5
	goto	INCCONTA		; Yes, zero TMR_100MS and inc 0,5 seconds counter
	goto	END_INT			; No, Exit interrupt
INCCONTA:
	movlw	D'0'			;
	movwf	TMR_100MS		; Zero 100 ms counter (reached 500ms)
	movlw	D'1'			; Verify if ...
	xorwf	T1_CONTA,W		; passed 1/2 second ...
	btfsc	STATUS,Z		; 
	goto	LEDON			; Led ONOFF is ON? Goto LEDON
LEDOFF:
	bcf		LED_ONOFF		; Turn OFF LEDON_OFF
	goto	LEDOUT			; ok, done...
LEDON:
	bsf		LED_ONOFF		; Turn ON LED_ONOFF
LEDOUT:
	decfsz	T1_CONTA,F		; Decrement the Counter rRegister
	goto	END_INT			; If diferent of zero, exit interrupt
	movlw	2				; 
	movwf	T1_CONTA		; Restart Counter Register with 2	
END_INT:
	swapf	STATUS_TEMP,W	; Restaure STATUS content...
	movwf	STATUS			; ... from STATUS_TEMP
	swapf	W_TEMP,F		; Restaure W content...
	swapf	W_TEMP,W		; ... from W_TEMP.
	retfie					; Return from interrupt

;/****************************************************************************
;* DESCRIPTION: Little Delay to let display show digit.
;* return:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

DELAY_MS:					; Delay routine - W is the number of ms of delay.
	movwf	TEMPO1			; put w in temp variable
	movlw	D'250'
	movwf	TEMPO0
	decfsz	TEMPO0,F
	goto	$-1
	decfsz	TEMPO1,F
	goto	$-5
	return

;/****************************************************************************
;* DESCRIPTION: binary_to_bcd - 8-bits
;* INPUT:		TEMPER  - 8-bit binary number
;* return:      CENTENA - the hundreds digit of the BCD conversion
;*				DEZENA  - the tens digits of the BCD conversion
;*				UNIDADE - the ones digits of the BCD conversion
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 
BINARY_TO_BCD:

	movwf	AUX				; sve the value to convert in AUX
	clrf	UNITS			;
	clrf	TENS			;
	clrf	HUNDREDS		; RESET variables
	movf	AUX,F			;
	btfsc	STATUS,Z		; Is the value to convert = 0?
	return					; Yes - Return
							; No
	incf	UNITS,F			; Increment unit
	movf	UNITS,W			;
	xorlw	0X0A			;
	btfss	STATUS,Z		; unit = 10d ?
	goto	$+3				; No
							; yes
	clrf	UNITS			; Reset unit
	incf	TENS,F			; Increment tens
	movf	TENS,W			;
	xorlw	0X0A			;
	btfss	STATUS,Z		; Tens = 10d ?
	goto	$+3				; No
							; Yes
	clrf	TENS			; Reset tens
	incf	HUNDREDS,F		; Increment hundreds
	decfsz	AUX,F			; End of conertion ?
	goto	$-.14			; No - go back to continue the convertion
	return					; Yes

;/****************************************************************************
;* DESCRIPTION: Check if any key is active (pressed)
;* return:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

CHECKKEYS:
	bsf		STATUS,RP0		; Bank 1
	bcf		STATUS,RP1		;

	movlw	B'00111100'		;
	movwf	TRISA			;
	movlw	B'11110000'		;
	movwf	TRISB			;

	movlw	B'00110100'
	movwf	WPUA
	bcf		STATUS,RP0		; Bank 2
	bsf		STATUS,RP1		;
	movlw	B'11110000'
	movwf	WPUB
	bsf		STATUS,RP0		; Bank 1
	bcf		STATUS,RP1		;
	bsf		OPTION_REG,7	; Set RABPU flag
	bcf		STATUS,RP0		; Bank 0
	bcf		STATUS,RP1		;

	;movf	PORTA, W		; Put PORTA in W
	;movwf	TMPPORTA		; Put it in temporary porta
	;movf	PORTB, W		; Put PORTB in W
	;movwf	TMPPORTB		; Put it in temporary portb

	btfss	KEY_UP			; KEY_UP pressed?
	call	FASTUP			; Yes
	btfss	KEY_DWN			; KEY_DWN pressed?
	call	FASTDWN			; Yes

	bsf		STATUS,RP0		; Bank 1
	bcf		STATUS,RP1		;
	movlw	B'00000000'
	movwf	WPUA
	bcf		STATUS,RP0		; Bank 2
	bsf		STATUS,RP1		;
	movlw	B'00000000'
	movwf	WPUB
	bsf		STATUS,RP0		; Bank 1
	bcf		STATUS,RP1		;
	bcf		OPTION_REG,7	; Set RABPU flag
	
	movlw	B'00001000'		;
	movwf	TRISA			;
	movlw	B'00000000'		;
	movwf	TRISB			;
	bcf		STATUS,RP0		; Bank 0
	bcf		STATUS,RP1		;

	return

;/****************************************************************************
;* DESCRIPTION: Process Key Up
;* return:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 
FASTUP:
	movlw	0x20
	call	BEEP

	decfsz	FILTER_KEYS,F
	btfss	SHOWTEMP		; Are we in temperature mode?
	goto	INCTIME			; No, we are in timer mode
	incf	TEMPER,F		; increment temperature
	goto	ENDKEYUP
INCTIME:
	incf	TIMER,F			; increment temperature
ENDKEYUP:
	return					; ends
FASTDWN:
	btfss	SHOWTEMP		; Are we in temperature mode?
	goto	DECTIME			; No, we are in timer mode
	decf	TEMPER,F		; decrement temperature
	goto	ENDKEYDWN		; we're done
DECTIME:
	decf	TIMER,F			; decrement timer
ENDKEYDWN:
	return					; return

;/****************************************************************************
;* DESCRIPTION: us delay
;* return:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

udelay						; delay W * 100 usec
	movwf	DCNT0
udelay0
	movlw	26
	movwf	DCNT1
udelay1
	decfsz	DCNT1,F
	goto	udelay1
	decfsz	DCNT0, F
	goto	udelay0
	return

;/****************************************************************************
;* DESCRIPTION: Beep routine
;* return:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

BEEP:
	movwf	BEEPCNT
BEEP0:
	bsf		BUZZER			; beepctl bit
	movlw	4
	call	udelay
	bcf		BUZZER			; beepctl bit
	movlw	4
	call	udelay
	decfsz	BEEPCNT, F
	goto	BEEP0
	return

;/****************************************************************************
;* DESCRIPTION: Program Initialization
;* return:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

START:
	bcf		STATUS,RP0		; Bank 2
	bsf		STATUS,RP1		;
	clrf	ANSEL			; digital I/O
	clrf	ANSELH			;	"	"	"
	bcf		STATUS,RP0		; Bank 2
	bsf		STATUS,RP1		;
	movlw	B'00000000'
	movwf	CM1CON0			; disable comparator
	movlw	B'00000000'		; Not necessary?
	movwf	CM2CON0			; disable 2nd comparator
	movlw	B'00000000'		; Not necessary?
	movwf	CM2CON1			; disable 3rd comparator
	bsf		STATUS,RP0		; Bank 1
	bcf		STATUS,RP1		;
	movlw	B'00001000'		;
	movwf	TRISA			;
	movlw	B'00000000'		;
	movwf	TRISB			;
	movlw	B'10000010'		;
	movwf	TRISC			;
	bcf		STATUS,RP0		; Bank 0
	bcf		STATUS,RP1		;
	bcf		SSPCON,SSPEN	; Disable Synchronous Serial Port

	; Turn off all reles, displays and leds
	bsf		LED_CMN			; Leds off
	bsf		DISP1			; Display1 off
	bsf		DISP2			; Display2 off
	bsf		DISP3			; Display3 off
	movlw	D'180'			; Init temperature with 180 degrees Celsius
	movwf	TEMPER			;
	movlw	D'40'			; Start TIMER with 40 minutes
	movwf	TIMER			;
	movlw	2				;
	movwf	T1_CONTA		;
	clrf	UNITS
	clrf	TENS
	clrf	HUNDREDS
	bsf		SHOWTEMP		; Start showing temperatue
	bcf		SHOWTIME		; Start not showing timer
	bcf		COOKINGMODE		; Cooking mode off
	bcf		PAUSEMODE		; Pause mode off
	bsf		TEMPMODE		; Temperature mode on
	bcf		TIMEMODE		; Time mode on
	bcf		LIGHTMODE		; Light off
	bcf		TOASTERMODE		; Toaster off

	; Init Timer1
	movlw	B'00000000'		; Configure TIMER1 for internal clock and prescaler 1:1
	movwf	T1CON			; Init Timer1
	movlw	0x18			;
	movwf	TMR1L			;
	movlw	0xFC			;
	movwf	TMR1H			; Init TIMER1 with decimal 64536 (interrupt every 1ms)

	bsf		STATUS,RP0		; Bank 1
	bcf		STATUS,RP1		;
	movlw   B'00000001'		; Enable TIMER1 Overflow Interrupt
	movwf	PIE1
	bcf		STATUS,RP0		; Bank 0
	bcf		STATUS,RP1		;
	clrf	PIR1			; Clear Peripheral Interrupt Flags
	bsf		INTCON, PEIE	; Enable Peripheral Interrupts
	bsf		T1CON, TMR1ON	; Start TMR1
	bsf		INTCON, GIE		; Enable Global Interrupts


;/****************************************************************************
;* DESCRIPTION: Main Program
;* return:      none
;* ALGORITHM:   none
;* NOTES:       none
;*****************************************************************************/ 

MAIN_LOOP:

	; Test displays mode:
	btfss	SHOWTEMP		; Are we in temperature mode?
	goto	USETIME			; No, we are in time mode
	movf	TEMPER,W		; Put temperature in W
	call	BINARY_TO_BCD	; Decode Temperature to bcd
	goto	DISPSHOW
USETIME:
	movf	TIMER,W			; Put timer value in W
	call	BINARY_TO_BCD	; Decode timer value to bcd

	; Displays code:
; Just for fun, the displays/leds timming:
;  _____   _____   _____   _____   _____   _____  __________________Off
;_|     |_|     |_|     |_|     |_|     |_|     | _______Display 1__On
;    _____   _____   _____   _____   _____   _____	________________Off
;  _|     |_|     |_|     |_|     |_|     |_|     | _____Display 2__On
;      _____   _____   _____   _____   _____   _____  ______________Off
;    _|     |_|     |_|     |_|     |_|     |_|     | ___Display 3__On
;        _____   _____   _____   _____   _____   _____  ____________Off
;      _|     |_|     |_|     |_|     |_|     |_|     |	_Leds_______On

DISPSHOW:
	bsf		LED_CMN			; Leds off
	movf	UNITS,W			; copy units to W
	call	DECOD_DISPLAY_A	; convert
	movwf	PORTA			; put it in PORTA
	movf	UNITS,W			; copy units to W
	call	DECOD_DISPLAY_B	; convert
	movwf	PORTB			; put it in PORTA
	bcf		DISP1			; display 1 on
	movlw	D'2'			;
	call	DELAY_MS		; delay 2 ms
	movf	TENS,W			; copy tens to W
	call	DECOD_DISPLAY_A	; convert
	bsf		DISP1			; display 1 off
	movwf	PORTA			; put it in PORTA
	movf	TENS,W			; copy tens to W
	call	DECOD_DISPLAY_B	; convert
	movwf	PORTB			; put it in PORTA
	bcf		DISP2			; display 2 on
	movlw	D'2'			;
	call	DELAY_MS		; delay 2 ms
	movf	HUNDREDS,W		; copy hundeds to W
	call	DECOD_DISPLAY_A	; convert
	bsf		DISP2			; display 2 off
	movwf	PORTA			; put it in PORTA
	movf	HUNDREDS,W		; copy hundeds to W
	call	DECOD_DISPLAY_B	; convert
	movwf	PORTB			; put it in PORTA
	bcf		DISP3			; display 3 on
	movlw	D'2'			;
	call	DELAY_MS		; delay 2 ms
	bsf		DISP3			; display 3 off

	; Leds code:
	bcf		LED_UP			; led UP ON
	bcf		LED_DWN			; led DWN ON
	btfss	TEMPMODE		; Show Teperature?
	goto	TEMPLEDOFF		; led TEMP OFF
	bcf		LED_TEMP		; led TEMP ON
	goto	TESTLEDTIMER	; Test Timer led
TEMPLEDOFF:
	bsf		LED_TEMP		; led TEMP OFF
TESTLEDTIMER:
	btfss	TIMEMODE		; Timer led ON?
	goto	TIMELEDOFF		; Timer led OFF
	bcf		LED_TIMER		; led TIMER ON
	goto	TESTLEDLIGHT	; Test Light led
TIMELEDOFF:
	bsf		LED_TIMER		; TIMER led OFF
TESTLEDLIGHT:
	btfss	LIGHTMODE		; Light led ON?
	goto	LIGHTLEDOFF		; led LIGHT OFF
	bcf		LED_LIGHT		; led LIGHT ON
	goto	TESTLEDTOASTER	; Test toaster led
LIGHTLEDOFF:
	bsf		LED_LIGHT		; led LIGHT OFF
TESTLEDTOASTER:
	btfss	TOASTERMODE		; toster mode on?
	goto	TOASTERLEDOFF	; led TOASTER OFF
	bcf		LED_TOASTER		; led TOASTER ON
	goto	LEDSON			; go get them!
TOASTERLEDOFF:
	bsf		LED_TOASTER		; led TOASTER OFF
LEDSON:
	bcf		LED_CMN			; Leds on
	movlw	D'2'			;
	call	DELAY_MS		; delay 2 ms
	bsf		LED_CMN			; Leds off

	; Keys code:
	call	CHECKKEYS		; Test keys

	goto	MAIN_LOOP		; Back to main loop

	END



